import 'dart:math';

import "package:benchmark_harness/benchmark_harness.dart";
import "package:poker/poker.dart";

class ExhaustiveBenchmark extends BenchmarkBase {
  ExhaustiveBenchmark() : super('Exhaustive');

  late Evaluator evaluator;

  late Iterable<EvaluationResult> iterable;

  late EvaluationResult result;

  // The benchmark code.
  @override
  void run() {
    for (final r in iterable) {
      result += r;
    }
  }

  @override
  void setup() {
    evaluator = ExhaustiveEvaluator(
      communityCards: CardSet.fromString("3c6dTs4d"),
      players: [
        HandRange.fromString("As3h"),
        HandRange.fromString("8d8h"),
        HandRange.fromString("AQs-ATsAKo-AJo44+"),
        HandRange.fromString("AQs-ATsAKo-AJo44+"),
      ],
    );
    iterable = evaluator.take(10000);
    result = EvaluationResult.empty(playerLength: 4);
  }

  @override
  void teardown() {
    print(result);
  }
}

class MontecarloBenchmark extends BenchmarkBase {
  MontecarloBenchmark() : super('Montecarlo');

  late Evaluator evaluator;

  late Iterable<EvaluationResult> iterable;

  late EvaluationResult result;

  // The benchmark code.
  @override
  void run() {
    for (final r in iterable) {
      result += r;
    }
  }

  @override
  void setup() {
    evaluator = MontecarloEvaluator(
      communityCards: CardSet.fromString("3c6dTs4d"),
      players: [
        HandRange.fromString("As3h"),
        HandRange.fromString("8d8h"),
        HandRange.fromString("AQs-ATsAKo-AJo44+"),
        HandRange.fromString("AQs-ATsAKo-AJo44+"),
      ],
    );
    iterable = evaluator.take(10000);
    result = EvaluationResult.empty(playerLength: 4);
  }

  @override
  void teardown() {
    print(result);
  }
}

// class EvaluationResultMatchupBenchmark extends BenchmarkBase {
//   EvaluationResultMatchupBenchmark() : super('EvaluationResultMatchup');

//   late Matchup matchup;

//   // The benchmark code.
//   @override
//   void run() {
//     EvaluationResult.fromMatchup(matchup);
//   }

//   @override
//   void setup() {
//     matchup = Matchup.showdown(
//       communityCards: CardSet.fromString("3c6dTs4d5d"),
//       playerCardPairs: [
//         CardSet.fromString("As3h"),
//         CardSet.fromString("8d8h"),
//         CardSet.fromString("AsTd"),
//       ],
//     );
//   }

//   @override
//   void teardown() {
//     print(EvaluationResult.fromMatchup(matchup));
//   }
// }

// class CardFromIntBenchmark extends BenchmarkBase {
//   CardFromIntBenchmark() : super('Card.fromInt()');

//   // The benchmark code.
//   @override
//   void run() {
//     // Card.fromInt(pow(2, Random().nextInt(52)).toInt());

//     // CardSet.full.elementAt(Random().nextInt(52));

//     // CardSet.full.contains(Card(rank: Rank.ace, suit: Suit.spade));
//     CardSet.empty.union(CardSet.full.union(CardSet.full));
//   }
// }

void main() {
  ExhaustiveBenchmark().report();
  MontecarloBenchmark().report();
  // EvaluationResultMatchupBenchmark().report();
  // CardFromIntBenchmark().report();
}
