import "dart:collection";
import "dart:math";
import "package:poker/poker.dart";
import "./evaluator.dart";

class MontecarloEvaluator
    with IterableMixin<EvaluationResult>
    implements Evaluator {
  MontecarloEvaluator({
    required this.communityCards,
    required this.players,
  });

  final CardSet communityCards;

  final List<HandRange> players;

  Iterator<EvaluationResult> get iterator => MontecarloEvaluationIterator(
        playerCardPairs: players.map((p) => p.toList()).toList(),
        initialDeck: CardSet.full.difference(communityCards),
        initialCommunityCards: communityCards,
      );

  @override
  int get length => -1;
}

class MontecarloEvaluationIterator implements Iterator<EvaluationResult> {
  MontecarloEvaluationIterator({
    required this.playerCardPairs,
    required this.initialDeck,
    required this.initialCommunityCards,
  })  : _random = Random(),
        _lastResult = null;

  final List<List<CardPair>> playerCardPairs;

  final CardSet initialDeck;

  final CardSet initialCommunityCards;

  final Random _random;

  EvaluationResult? _lastResult;

  @override
  EvaluationResult get current => _lastResult!;

  @override
  bool moveNext() {
    final chosenPlayerCardPairs =
        playerCardPairs.map((cps) => cps[_random.nextInt(cps.length)]).toList();

    CardSet deck = initialDeck;
    CardSet communityCards = initialCommunityCards;
    bool isPossible = true;

    for (final cardPair in chosenPlayerCardPairs) {
      if (!deck.contains(cardPair.first) || !deck.contains(cardPair.last)) {
        isPossible = false;

        break;
      }

      deck = deck.difference(cardPair);
    }

    if (isPossible) {
      for (int i = communityCards.length; i < 5;) {
        final card = Card.fromInt(pow(2, _random.nextInt(52)) as int);

        if (deck.contains(card)) {
          deck = deck.difference(CardSet.single(card));
          communityCards = communityCards.union(CardSet.single(card));

          i += 1;
        }
      }

      final matchup = Matchup.showdown(
        communityCards: communityCards,
        playerCardPairs: chosenPlayerCardPairs,
      );

      _lastResult = EvaluationResult.fromMatchup(matchup);
      // _lastResult =
      //     EvaluationResult.empty(playerLength: chosenPlayerCardPairs.length);
    } else {
      _lastResult =
          EvaluationResult.empty(playerLength: chosenPlayerCardPairs.length);
    }

    return true;
  }
}
