import 'dart:math';

import "package:poker/poker.dart";

class EvaluationResult {
  EvaluationResult({
    required this.tries,
    required this.games,
    required this.playerWons,
  });

  EvaluationResult.empty({required int playerLength})
      : tries = 0,
        games = 0,
        playerWons = List.filled(playerLength, 0);

  factory EvaluationResult.fromMatchup(Matchup matchup) {
    MadeHand? bestHand = null;
    List<int> bestPlayerIndexes = [];

    for (int i = 0; i < matchup.playerMadeHands.length; ++i) {
      final madeHand = matchup.playerMadeHands[i];

      if (madeHand.power > (bestHand?.power ?? -1)) {
        bestHand = madeHand;
        bestPlayerIndexes = [i];
      } else if (madeHand.power == bestHand?.power) {
        bestPlayerIndexes.add(i);
      }
    }

    return EvaluationResult(
      tries: 1,
      games: 1,
      playerWons: List.generate(
          matchup.playerMadeHands.length,
          (playerIndex) => bestPlayerIndexes.contains(playerIndex)
              ? 1 / bestPlayerIndexes.length
              : 0),
    );
  }

  /**
   *
   */
  final int tries;

  /**
   *
   */
  final int games;

  final List<double> playerWons;

  @override
  String toString() =>
      "EvaluationResult<tries: $tries, games: $games, playerWons: $playerWons>";

  EvaluationResult operator +(EvaluationResult other) {
    List<double> nextPlayerWons = playerWons.toList();

    for (int i = 0; i < min(playerWons.length, other.playerWons.length); ++i) {
      nextPlayerWons[i] += other.playerWons[i];
    }

    return EvaluationResult(
      tries: tries + other.tries,
      games: games + other.games,
      playerWons: nextPlayerWons,
    );
  }
}

// class EvaluationResultPlayer {
//   EvaluationResultPlayer({
//     required this.won,
//     required this.tie,
//   });

//   EvaluationResultPlayer.empty({required int playerLength})
//       : won = 0,
//         tie = List.filled(playerLength - 1, 0);

//   final int won;

//   final List<int> tie;

//   @override
//   String toString() =>
//       "EvaluationResultPlayer<won: $won, tie: ${tie.fold<int>(0, (t, v) => t + v)}>";

//   EvaluationResultPlayer operator +(EvaluationResultPlayer other) {
//     final nextTie = tie.toList();

//     for (int i = 0; i < min(tie.length, other.tie.length); ++i) {
//       nextTie[i] += other.tie[i];
//     }

//     return EvaluationResultPlayer(
//       won: won + other.won,
//       tie: nextTie,
//     );
//   }
// }
